@@ -31,7 +31,7 @@ use Data::FormValidator::Constraints (qw/:validators :matchers/);
 
 use vars qw( $VERSION $AUTOLOAD @ISA @EXPORT_OK %EXPORT_TAGS );
 
-$VERSION = '3.00';
+$VERSION = '3.49_1';
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -125,7 +125,7 @@ Data::FormValidator's main aim is to make input validation expressible in a
 simple format.
 
 Data::FormValidator lets you define profiles which declare the
-required  and optional fields and any constraints they might have.
+required and optional fields and any constraints they might have.
 
 The results are provided as an object which makes it easy to handle 
 missing and invalid results, return error messages about which constraints
@@ -168,9 +168,11 @@ sub new {
 
 =head2 check()
 
-C<check> is the recommended method to use to validate forms. It returns it's results as
+ my $results = Data::FormValidator->check(\%input_hash, \%dfv_profile);
+
+C<check> is the recommended method to use to validate forms. It returns it's results as a
 L<Data::FormValidator::Results|Data::FormValidator::Results> object.  A
-deprecated method C<validate> is also available, returning it's results in
+deprecated method C<validate> is also available, returning it's results as an
 array described below.
 
  use Data::FormValidator;
@@ -179,9 +181,10 @@ array described below.
 Here, C<check()> is used as a class method, and takes two required parameters. 
 
 The first a reference to the data to be be validated. This can either be a hash
-reference, or a CGI or Apache::Request object.  Note that if you use a hash
-reference, multiple values for a single key should be presented as an array
-reference.
+reference, or a CGI.pm-like object. In particular, the object must have a param()
+method that works like the one in CGI.pm does. CGI::Simple and Apache::Request
+objects are known to work in particular. Note that if you use a hash reference,
+multiple values for a single key should be presented as an array reference.
 
 The second argument is a reference to the profile you are validating.
 
@@ -197,14 +200,14 @@ syntax, but returns a four element array, described as follows
 
 =item valids
 
-This is an hash reference to the valid fields which were submitted in
+This is a hash reference to the valid fields which were submitted in
 the data. The data may have been modified by the various filters specified.
 
 =item missings
 
 This is a reference to an array which contains the name of the missing
 fields. Those are the fields that the user forget to fill or filled
-with space. These fields may comes from the I<required> list or the
+with spaces. These fields may comes from the I<required> list or the
 I<dependencies> list.
 
 =item invalids
@@ -276,14 +279,14 @@ validation profile will completely overwrite your default value.
 This means you can't define two keys for C<constraint_regexp_map> and expect
 they will always be there. This kind of feature may be added in the future.
 
-The exception here is are definitions for your C<msgs> key. You will safely  
+The exception here is definitions for your C<msgs> key. You will safely  
 be able to define some defaults for this key and not have them entirely clobbered 
 just because C<msgs> was defined in a validation profile.
 
 One way to use this feature is to create your own sub-class that always provides
 your defaults to C<new()>. 
 
-Another option to create your own wrapper routine which provides these defaults to 
+Another option is to create your own wrapper routine which provides these defaults to 
 C<new()>.  Here's an example of a routine you might put in a
 L<CGI::Application|CGI::Application> super-class to make use of this feature:
 
@@ -391,30 +394,30 @@ examples of each.
 =head2 required
 
 This is an array reference which contains the name of the fields which are
-required. Any fields in this list which are not present, or contain only
-spaces.  will be reported as missing.
-
+required. Any fields in this list which are not present or contain only
+spaces will be reported as missing.
 
 =head2 required_regexp
 
  required_regexp => qr/city|state|zipcode/,
 
-This is a regular expression used to specify additional fieds which are
+This is a regular expression used to specify additional fields which are
+required.
+
+=head2 require_some
 
  require_some => {
     # require any two fields from this group
     city_or_state_or_zipcode => [ 2, qw/city state zipcode/ ], 
  }
 
-This is a reference to a hash which defines groups of fields where 
-1 or more field from the group should be required, but exactly
-which fields doesn't matter. The keys in the hash are the group names. 
-These are returned as "missing" unless the required number of fields
-from the group has been filled in. The values in this hash are
-array references. The first element in this hash should be the 
-number of fields in the group that is required. If the first
-first field in the array is not an a digit, a default of "1" 
-will be used. 
+This is a reference to a hash which defines groups of fields where 1 or more
+fields from the group should be required, but exactly which fields doesn't
+matter. The keys in the hash are the group names.  These are returned as
+"missing" unless the required number of fields from the group has been filled
+in. The values in this hash are array references. The first element in this
+array should be the number of fields in the group that is required. If the
+first field in the array is not an a digit, a default of "1" will be used. 
 
 =head2 optional
 
@@ -422,14 +425,14 @@ will be used.
 
 This is an array reference which contains the name of optional fields.
 These are fields which MAY be present and if they are, they will be
-check for valid input. Any fields not in optional or required list
+checked for valid input. Any fields not in optional or required list
 will be reported as unknown.
 
 =head2 optional_regexp
 
  optional_regexp => qr/_province$/,
 
-This is a regular expression used to specify additional fieds which are
+This is a regular expression used to specify additional fields which are
 optional. For example, if you wanted all fields names that begin with I<user_> 
 to be optional, you could use the regular expression, /^user_/
 
@@ -468,6 +471,7 @@ This is a hash reference which contains information about groups of
 interdependent fields. The keys are arbitrary names that you create and
 the values are references to arrays of the field names in each group. 
 
+
 =head2 defaults
 
  defaults => {
@@ -475,11 +479,12 @@ the values are references to arrays of the field names in each group.
  },
 
 This is a hash reference where keys are field names and 
-values are defaults to use if input for th e field is missing. 
+values are defaults to use if input for the field is missing. 
 
 The defaults are set shortly before the constraints are applied, and
 will be returned with the other valid data.
 
+
 =head2 filters
 
  # trim leading and trailing whitespace on all fields
@@ -499,7 +504,7 @@ See Data::FormValidator::Filters for details on the built-in filters.
 =head2 field_filters
 
  field_filters => { 
- 	cc_no => "digit"
+ 	cc_no => ['digit'],
  },
 
 A hash ref with field names and keys. Values are array references
@@ -511,7 +516,7 @@ See Data::FormValidator::Filters for details on the built-in filters.
 
  field_filter_regexp_map => {
  	# Upper-case the first letter of all fields that end in "_name"
- 	qr/_name$/	=> 'ucfirst',
+ 	qr/_name$/	=> ['ucfirst'],
  },
 
 This is a hash reference where the keys are the regular expressions to
@@ -533,29 +538,39 @@ match a regular expression.
 A hash ref which contains the constraints that
 will be used to check whether or not the field contains valid data.
 
-The keys in this hash are field names. The values can any of the following:
+The keys in this hash are field names. The values can be any of the following:
 
 =over 
 
 =item o
 
-the name of a builtin constraint function (see below)
+A named constraint. 
 
 B<Example>: 
 
  my_zipcode_field 	=> 'zip',
 
+See L<Data::FormValidator::Constraints> for the details of which
+built-in constraints that are available.
+
+
 =item o 
 
 a perl regular expression
 
 B<Example>: 
 
- my_zipcode_field   => '/^\d{5}$/', # match exactly 5 digits
+ my_zipcode_field   => qr/^\d{5}$/, # match exactly 5 digits
+
+If this field is named in the C<untaint_constraint_fields>, or 
+C<untaint_all_constraints> is effective, be aware of the following: If you
+write your own regular expressions and only match part of the string then
+you'll only get part of the string in the valid hash. It is a good idea to
+write you own constraints like /^regex$/. That way you match the whole string.
 
 =item o
 
-a subroutine reference
+a subroutine reference, to supply custom code
 
 This will check the input and return true or false depending on the input's validity.
 By default, the constraint function takes one parameter, the field to be
@@ -570,12 +585,39 @@ B<Examples>:
  # OR you can reference a subroutine, which should work like the one above
  my_zipcode_field => \&my_validation_routine, 
 
+=item o
+
+a hash reference, to name a constraint or supply multiple parameters.
+
+ # supply multiple parameters
+ cc_no  => {  
+ 	constraint  => "cc_number",
+ 	params	     => [ qw( cc_no cc_type ) ],
+ },
+
+ # name a constraint, useful for returning error messages
+ last_name => {
+ 	name => "ends_in_name",
+	constraint => qr/_name$/,
+ },
+
+Using the hash reference for a constraint allows the possibility to
+pass in multiple arguments to the constraint, and to provide a name for a
+constraint that doesn't have one. The name can be used in the error message
+system to return a custom error message for this constraint. In some cases you
+will want to use C<constraint_method> instead of C<constraint>.  Consult the
+documentation for the constraint you are using to see which is correct for
+that case.  If in doubt, use C<constraint>.
+
+For details see L<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
+
 =item o 
 
 an array reference
 
 An array reference is used to apply multiple constraints to a single
-field. See L<MULTIPLE CONSTRAINTS> below.
+field. Any of the above options are valid entries the array.
+See L<MULTIPLE CONSTRAINTS> below.
 
 =back
 
@@ -597,16 +639,12 @@ field for each regular expression that matches.
 
  untaint_all_constraints => 1,
 
-If this field is set all form data that passes a constraint will be untainted.
+If this field is set, all form data that passes a constraint will be untainted.
 The untainted data will be returned in the valid hash.  Untainting is based on
-the pattern match used by the constraint.  Note that some validation routines
+the pattern match used by the constraint.  Note that some constraint routines
 may not provide untainting.
 
-If you write your own regular expressions and only match part of the string
-then you'll only get part of the string in the valid hash. It is a good idea to
-write you own constraints like /^regex$/. That way you match the whole string.
-
-See L<WRITING YOUR OWN VALIDATION ROUTINES> in the Data::FormValidator::Results
+See L<WRITING YOUR OWN CONSTRAINT ROUTINES> in the Data::FormValidator::Constraints
 documention for more information.
 
 This is overridden by C<untaint_constraint_fields>
@@ -619,7 +657,7 @@ Specifies that one or more fields will be untainted if they pass their
 constraint(s). This can be set to a single field name or an array reference of
 field names. The untainted data will be returned in the valid hash. 
 
-This is overrides the untaint_all_constraints flag.
+This overrides the untaint_all_constraints flag.
 
 =head2 missing_optional_valid
 
@@ -638,38 +676,33 @@ set back to "blank" may fail to get updated.
  # load all the constraints from these modules
  validator_packages => [qw(Data::FormValdidator::Constraints::Upload)],
 
-This key is used to define other packages which contain validation routines.
+This key is used to define other packages which contain constraint routines.
 Set this key to a single package name, or an arrayref of several. All of its
-validation routines. will become available for use.  beginning with 'match_'
-and 'valid_' will be imported into Data::FormValidator.  This lets you
-reference them in a constraint with just their name, just like built-in
-routines .  You can even override the provided validators.
+constraint routines  beginning with 'match_' and 'valid_' will be imported into
+Data::FormValidator.  This lets you reference them in a constraint with just
+their name, just like built-in routines.  You can even override the provided
+validators.
 
-See L<WRITING YOUR OWN VALIDATION ROUTINES> in the Data::FormValidator::Results
-documention for more information
+See L<WRITING YOUR OWN CONSTRAINT ROUTINES> in the Data::FormValidator::Constraints
+documentation for more information
 
 =head2 msgs
 
- msgs =>{},
+B<NOTE:> This part of the interface is newer and may change.  Use in production
+code at your own caution. Contact the maintainer with any questions or
+suggestions.
 
-B<NOTE:> This part of the interface is still experimental and may change.  Use
-in production code at your own caution. Contact the maintainer with any
-questions or suggestions.
+This key is used to define parameters related to formatting error messages
+returned to the user.
 
-This key is used to transform the output of the invalid and missing
-return values into a single hash reference which has the field names as keys
-and error messages as values. By default, invalid fields have the message
-"Invalid" associated with them while missing fields have the message "Missing"
-associated with them. In the simplest case, this key can simply be defined as a
-reference to a an empty hash, like this:
+By default, invalid fields have the message "Invalid" associated with them
+while missing fields have the message "Missing" associated with them. 
 
-B<Example>:
+In the simplest case, nothing needs to be defined here, and the default values
+will be used. 
 
- msgs =>{}
-
-This will cause the default messages to be used for missing and invalid fields. Some
-default formatting will also be applied, designed for display in an XHTML web
-page. That formatting is as followings:
+The default formatting applied is designed for display in an XHTML web page.
+That formatting is as followings:
 
 	<span style="color:red;font-weight:bold"><span id="dfv_errors">* %s</span></span>
 
@@ -705,7 +738,7 @@ as providing custom messages per field, and handling multiple constraints:
                      # ...
      },
  
- 	# This token will be included in the hash if they are 
+ 	# This token will be included in the hash if there are 
  	# any errors returned. This can be useful with templating
  	# systems like HTML::Template
  	# The 'prefix' setting does not apply here.
@@ -713,12 +746,12 @@ as providing custom messages per field, and handling multiple constraints:
  	any_errors => 'some_errors',
  }
 
-The hash that's prepared can be retreived through the C<msgs> method
+The hash that's prepared can be retrieved through the C<msgs> method
 described in the L<Data::FormValidator::Results> documentation.
 
 =head2 debug
 
-This method is used to print details about is going on to STDERR.
+This method is used to print details about what is going on to STDERR.
 
 Currently only level '1' is used. It provides information about which 
 fields matched constraint_regexp_map. 
@@ -750,18 +783,18 @@ Deprecated, but supported
 
 =head1 VALIDATING INPUT BASED ON MULTIPLE FIELDS
 
-You can pass more than one value into a validation routine.  For that, the
+You can pass more than one value into a constraint routine.  For that, the
 value of the constraint should be a hash reference. If you are creating your
-own routines, be sure to read the section labeled L<WRITING YOUR OWN VALIDATION ROUTINES>, 
-in the Data::FormValidator::Results documentation.
-It describes a newer and more flexible syntax. 
+own routines, be sure to read the section labeled L<WRITING YOUR OWN VALIDATION
+ROUTINES>, in the Data::FormValidator::Results documentation.  It describes a
+newer and more flexible syntax. 
 
 Using the original syntax, one key should be named C<constraint> and should
 have a value set to the reference of the subroutine or the name of a built-in
 validator.  Another required key is I<params>. The value of the I<params> key
 is a reference to an array of the other elements to use in the validation. If
-the element is a scalar, it is assumed to a field name. If the value is a
-reference, The reference is passed directly to the routine. Don't forget to
+the element is a scalar, it is assumed to be a field name. If the value is a
+reference, the reference is passed directly to the routine. Don't forget to
 include the name of the field to check in that list, if you are using this syntax.
 
 B<Example>:
@@ -813,7 +846,7 @@ sub load_profiles {
     return if $self->{profiles} and $self->{profiles_mtime} <= $mtime;
 
     $self->{profiles} = do $file;
-    die "Input profiles didn't return an hash ref\n"
+    die "Input profiles didn't return a hash ref\n"
       unless ref $self->{profiles} eq "HASH";
 
     $self->{profiles_mtime} = $mtime;
@@ -884,7 +917,7 @@ L<Data::FormValidator::Results|Data::FormValidator::Results>
 
 B<A sample application by the maintainer:> 
 
-Validationg Web Forms with Perl, L<http://mark.stosberg.com/Tech/perl/form-validation/>
+Validating Web Forms with Perl, L<http://mark.stosberg.com/Tech/perl/form-validation/>
 
 B<Related modules:>
 
@@ -893,21 +926,26 @@ L<Data::FormValidator::Tutorial|Data::FormValidator::Tutorial>
 L<CGI::Application::ValidateRM|CGI::Application::ValidateRM>, a
 CGI::Application & Data::FormValidator glue module
 
-L<Params::Validate |Params::Validate> looks like a better choice for validating function parameters.  
+L<Params::Validate|Params::Validate> looks like a better choice for validating function parameters.  
 
 L<Regexp::Common|Regexp::Common>,
 L<Data::Types|Data::Types>,
 L<Data::Verify|Data::Verify>,
+L<Email::Valid|Email::Valid>,
 L<String::Checker|String::Checker>,
 L<CGI::ArgChecker|CGI::ArgChecker>,
 L<CGI::FormMagick::Validator|CGI::FormMagick::Validator>,
 L<CGI::Validate|CGI::Validate>
 
-
 B<Document Translations:>
 
 Japanese: L<http://perldoc.jp/docs/modules/>
 
+B<Distributions which include Data::formValidator> 
+
+FreeBSD includes a port named B<p5-Data-FormValidator>
+Debian GNU/Linux includes a port named B<libdata-formvalidator-perl>
+
 =head1 CREDITS
 
 Some of those input validation functions have been taken from MiniVend
