@@ -11,14 +11,18 @@
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms same terms as perl itself.
 #
+#  $Header$
 use strict;
 
 package Data::FormValidator::Results;
 
 use Data::FormValidator::Filters qw/:filters/;
 use Data::FormValidator::Constraints (qw/:validators :matchers/);
+use vars qw/$AUTOLOAD $VERSION/;
 use Symbol;
 
+$VERSION = 3.49_1;
+
 =pod
 
 =head1 NAME
@@ -27,7 +31,7 @@ Data::FormValidator::Results - Object which contains the results of an input val
 
 =head1 SYNOPSIS
 
-    my $results = $validator->check( \%fdat, "customer_infos" );
+ 	my $results = Data::FormValidator->check(\%input_hash, \%dfv_profile);
 
     # Print the name of missing fields
     if ( $results->has_missing ) {
@@ -88,9 +92,11 @@ sub _process {
 	# msgs() method will need access to the profile
 	$self->{profile} = $profile;
 
+	my %imported_validators;
+
     # import valid_* subs from requested packages
 	foreach my $package (_arrayify($profile->{validator_packages})) {
-		if ( !exists $profile->{imported_validators}{$package} ) {
+		if ( !exists $imported_validators{$package} ) {
 			eval "require $package";
 			if ($@) {
 				die "Couldn't load validator package '$package': $@";
@@ -107,7 +113,7 @@ sub _process {
 					*{qualify_to_ref($sub)} = $subref;
 				}
 			}
-			$profile->{imported_validators}{$package} = 1;
+			$imported_validators{$package} = 1;
 		}
 	}
 
@@ -118,7 +124,7 @@ sub _process {
 			$filter = (ref $filter ? $filter : *{qualify_to_ref("filter_$filter")}{CODE}) ||
 				die "No filter found named: '$filter'";
 			foreach my $field ( keys %valid ) {
-				# apply filter, modifying %valid by reference
+				# apply filter, modifying %valid by reference, skipping undefined values
 				_filter_apply(\%valid,$field,$filter);
 			}
 		}	
@@ -190,12 +196,12 @@ sub _process {
 		if (ref $valid{$field}) {
 			if ( ref $valid{$field} eq 'ARRAY' ) {
 				for (my $i = 0; $i < scalar @{ $valid{$field} }; $i++) {
-					$valid{$field}->[$i] = undef unless length $valid{$field}->[$i];
+					$valid{$field}->[$i] = undef unless (defined $valid{$field}->[$i] and length $valid{$field}->[$i]);
 				}
 			}
 		}
 		else {
-			delete $valid{$field} unless length $valid{$field};
+			delete $valid{$field} unless (defined $valid{$field} and length $valid{$field});
 		}
 	}
 
@@ -551,13 +557,10 @@ sub msgs {
 		die "$0: parameter passed to msgs must be a hash ref";
 	}
 
-#	unless (ref $self->{profile}->{msgs} eq 'HASH') {
-#		warn "msgs without called without msgs definition in validation profile";
-#		return undef;
-#	}
 
 	# Allow msgs to be called more than one to accumulate error messages
 	$self->{msgs} ||= {};
+	$self->{profile}->{msgs} ||= {};
 	$self->{msgs} = { %{ $self->{msgs} }, %$controls };
 
 	my %profile = (
@@ -601,110 +604,85 @@ sub msgs {
 
 =pod
 
-=head1 WRITING YOUR OWN VALIDATION ROUTINES
+=head1 meta()
 
-It's easy to create your own module of validation routines. The easiest approach
-to this may be to check the source code of the Data::FormValidator module for example
-syntax. Also notice the C<validator_packages> option in the input profile.
+In a few cases, a constraint may discover meta data that is useful
+to access later. For example, when using L<Data::FormValidator::Constraints::Upload>, several bits of meta data are discovered about files in the process
+of validating. These can include "bytes", "width", "height" and "extension".
+The C<meta()> function is used by constraint methods to set this data. It's
+also used to access this data. Here are some examples.
 
-You will find that validation routines are named two ways. Some are named with
-the prefix C<match_> while others start with C<valid_>. The difference is that the
-C<match_ routines> are built to untaint the data and routine a safe version of
-it if it validates, while C<valid_> routines simply return a true value if the
-validation succeeds and false otherwise.
+ # return all field names that have meta data
+ my @fields = $results->meta();
 
-It is preferable to write "match" routines that untaint data for the extra security
-benefits. Plus, Data::FormValidator will AUTOLOAD a "valid_" version if anyone tries to
-use it, so you only need to write one routine to cover both cases. 
+ # To retrieve all meta data for a field:
+ $meta_href = $results->meta('img');
+ 
+ # Access a particular piece: 
+ $width = $results->meta('img')->{width};
+ 
+Here's how to set some meta data. This is useful to know if you are
+writing your own complex constraint.
 
-Usually validation routines only need one input, the value being specified. However,
-sometimes more than one value is needed. For that, the following syntax is
-recommended for calling the routines:
+	$self->meta('img', {
+		width  => '50',
+		height => '60',
+	});
 
-B<Example>:
+This function does not currently multi-valued fields. If it 
+does in the future, the above syntax will still work..
 
-		image_field  => {  
-			constraint_method  => 'max_image_dimensions',
-			params => [\100,\200],
-		},
+=cut
 
-Using this syntax, the first parameter that will be passed to the routine is
-the Data::FormValidator object. The remaining parameters will come from the
-C<params> array. Strings will be replaced by the values of fields with the same names,
-and references will be passed directly.
+sub meta {
+	my $self  = shift;
+	my $field = shift;
+	my $data  = shift;
 
-A couple of of useful methods to use on the Data::FormValidator::Results object  are
-available to you to use inside of your routine.
+	# initialize if it's the first call
+	$self->{__META} ||= {};
 
-=over 4
+	if ($data) {
+		(ref $data eq 'HASH') or die 'meta: data passed not a hash ref'; 
+        $self->{__META}{$field} = $data;
+	}
 
-=item get_input_data
 
-Returns the raw input data. This may be a CGI object if that's what 
-was used in the validation routine. 
+	# If we are passed a field, return data for that field
+	if ($field) {
+		return $self->{__META}{$field};
+	}
+	# Otherwise return a list of all fields that have meta data
+	else {
+		return keys %{ $self->{__META} };
+	}
+}
 
-B<Example>
-
- my $data = $self->get_input_data;
-
-=back
-
-=cut 
+# These are documented in ::Constraints, in the section
+# on writing your own routines. It was more intuitive
+# for the user to look there. 
 
 sub get_input_data {
 	my $self = shift;
 	return $self->{__INPUT_DATA};
 }
 
-=pod
-
-=over 4
-
-=item get_current_constraint_field
-
-Returns the name of the current field being tested in the constraint.
-
-B<Example>:
-
- my $field = $self->get_current_constraint_field;
-
-This reduces the number of parameters that need to be passed into the routine
-and allows multi-valued constraints to be used with C<constraint_regexp_map>.
-
-=back
-
-For complete examples of multi-valued constraints, see L<Data::FormValidator::Constraints::Upload>
-
-=cut
-
 sub get_current_constraint_field {
 	my $self = shift;
 	return $self->{__CURRENT_CONSTRAINT_FIELD};
 }
-=pod
-
-=over 4
-
-=item get_current_constraint_value
-
-Returns the name of the current value being tested in the constraint.
-
-B<Example>:
-
- my $value = $self->get_current_constraint_value;
-
-This reduces the number of parameters that need to be passed into the routine
-and allows multi-valued constraints to be used with C<constraint_regexp_map>.
-
-=back
-
-=cut
 
 sub get_current_constraint_value {
 	my $self = shift;
 	return $self->{__CURRENT_CONSTRAINT_VALUE};
 }
 
+sub get_current_constraint_name {
+	my $self = shift;
+	return $self->{__CURRENT_CONSTRAINT_NAME};
+}
+
+
 # INPUT: prefix_string, hash reference
 # Copies the hash and prefixes all keys with prefix_string
 # OUTPUT: hash refence
@@ -726,14 +704,29 @@ sub prefix_hash {
 # (which must start with a slash or "m", not a paren)
 sub _create_sub_from_RE {
 	my $re = shift || return undef;
+	my $untaint_this = shift;
+
+	# This looks like VooDoo to me, but it works. Simplications welcome. -mls 05/26/03 
+	my $return_code = ($untaint_this) ? '; return ($& =~ m/(.*)/s)[0] if defined($`);' : '';
+
 	my $sub;
-	if ($re =~ /^\(/) {
-		$sub = sub { $_[0] =~ $re };
+	# If it's "qr" style
+	if (substr($re,0,1) eq '(') {
+		$sub = sub { 
+			my $match = $_[0] =~ $re; 
+			if ($untaint_this && defined $-[0]) {
+				return (substr($_[0], $-[0], $+[0] - $-[0]) =~ m/(.*)/s)[0];
+			}
+			else {
+				return $match;
+			}
+		};
+
 	}
 	else {
-		$sub = eval 'sub { $_[0] =~ '.$re. '}';
-		die "Error compiling regular expression $re: $@" if $@;
+		$sub = eval 'sub { $_[0] =~ '.$re.$return_code. '}';
 	}
+	die "Error compiling regular expression $re: $@" if $@;
 	return $sub;
 }
 
@@ -766,16 +759,18 @@ sub _arrayify {
 }
 
 # apply filter, modifying %valid by reference
+# We don't bother trying to filter undefined fields.
+# This prevents warnings from Perl. 
 sub _filter_apply {
 	my ($valid,$field,$filter) = @_;
 	die 'wrong number of arguments passed to _filter_apply' unless (scalar @_ == 3);
 	if (ref $valid->{$field} eq 'ARRAY') {
 		for (my $i = 0; $i < @{ $valid->{$field} }; $i++) {
-			$valid->{$field}->[$i] = $filter->( $valid->{$field}->[$i] );
+			$valid->{$field}->[$i] = $filter->( $valid->{$field}->[$i] ) if defined $valid->{$field}->[$i];
 		}
 	}
 	else {
-		$valid->{$field} = $filter->( $valid->{$field} );
+		$valid->{$field} = $filter->( $valid->{$field} ) if defined $valid->{$field};
 	}
 }
 
@@ -798,11 +793,9 @@ sub _constraint_hash_build {
 	}
 
 	# Check for regexp constraint
-	if ( $c->{constraint} =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ ) {
-		#If untainting return the match otherwise return result of match
-               my $return_code = ($untaint_this) ? 'return (substr($_[0], $-[0], $+[0] - $-[0]) =~ m/(.*)/s)[0] if defined($-[0]);' : '';
-		$c->{constraint} = eval 'sub { $_[0] =~ '. $c->{constraint} . ';' . $return_code . '}';
-		die "Error compiling regular expression $c->{constraint}: $@" if $@;
+	if ((ref $c->{constraint} eq 'Regexp')
+			or ( $c->{constraint} =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ )) {
+		$c->{constraint} = _create_sub_from_RE($c->{constraint},$untaint_this);
 	}
 	# check for code ref
 	elsif (ref $c->{constraint} eq 'CODE') {
@@ -811,11 +804,25 @@ sub _constraint_hash_build {
 	else {
 		# provide a default name for the constraint if we don't have one already
 		$c->{name} ||= $c->{constraint};
+
+		# Save the current constraint name for later
+		$self->{__CURRENT_CONSTRAINT_NAME} = $c->{name};
 		
 		#If untaint is turned on call match_* sub directly. 
 		if ($untaint_this) {
-			$c->{constraint} = *{qualify_to_ref("match_$c->{constraint}")}{CODE} ||
-				die "No untainting constraint found named '$c->constraint'";
+			my $routine = 'match_'.$c->{constraint};			
+			my $match_sub = *{qualify_to_ref($routine)}{CODE};
+			if ($match_sub) {
+				$c->{constraint} = $match_sub; 
+			}
+			# If the constraint name starts with RE_, try looking for it in the Regexp::Common package
+			elsif ($c->{constraint} =~ m/^RE_/) {
+				$c->{is_method} = 1;
+				$c->{constraint} = eval 'sub { &_create_regexp_common_constraint(@_)}' 
+					|| die "could not create Regexp::Common constraint: $@";
+			} else {
+				die "No untainting constraint found named $c->{constraint}";
+			}
 		}
 		else {
 			# try to use match_* first
@@ -828,6 +835,12 @@ sub _constraint_hash_build {
 			elsif (my $valid_sub = *{qualify_to_ref('valid_'.$c->{constraint})}{CODE}) {
 				$c->{constraint} = $valid_sub;
 			}
+			# Load it from Regexp::Common 
+			elsif ($c->{constraint} =~ m/^RE_/) {
+				$c->{is_method} = 1;
+				$c->{constraint} = eval 'sub { return defined &_create_regexp_common_constraint(@_)}' ||
+					die "could not create Regexp::Common constraint: $@";
+			}
 			else {
 				die "No constraint found named '$c->{name}'";
 			}
@@ -874,16 +887,18 @@ sub _constraint_check_match {
 	}
 }
 
-# Figure out whether the data is a hash reference of a CGI or Apache::Request object and return it has a hash
+# Figure out whether the data is a hash reference of a param-capable object and return it has a hash
 sub _get_data {
 	my ($self,$data) = @_;
 	$self->{__INPUT_DATA} = $data;
 	require UNIVERSAL;
-	if (UNIVERSAL::isa($data,'CGI') || UNIVERSAL::isa($data,'Apache::Request') ) {
+
+    # This checks whether we have an object or not.
+    if (UNIVERSAL::isa($data,'UNIVERSAL') ) {
 		my %return;
 		# make sure object supports param()
 		defined($data->UNIVERSAL::can('param')) or
-		croak("Data::FormValidator->validate called with CGI or Apache::Request object which lacks a param() method!");
+		croak("Data::FormValidator->validate() or check() called with an object which lacks a param() method!");
 		foreach my $k ($data->param()){
 			# we expect param to return an array if there are multiple values
 			my @v = $data->param($k);
@@ -898,6 +913,20 @@ sub _get_data {
 }
 
 
+sub _create_regexp_common_constraint  {
+	require Regexp::Common;
+	import  Regexp::Common 'RE_ALL';
+	my $self = shift;
+	my $re_name = $self->get_current_constraint_name;
+	# deference all input
+	my @params = map {$_ = $$_ if ref $_ }  @_;
+
+	no strict "refs";
+	my $re = &$re_name(-keep=>1,@params) || die 'no matching Regexp::Common routine found';
+	return ($self->get_current_constraint_value =~ qr/^$re$/) ? $1 : undef; 
+}
+
+
 1;
 
 __END__
